;*************************************************************************************
;*************************************************************************************
;
; Testprogramm zum Entwickeln der 
; Bibliothek zum Verwenden eines externen EEPROMs vom Typ 24C64
; (Header Datei: i2c_inc_h.asm)
; source: i2c_inc.asm 
; Rev. 1.1 von 2009-12-27
; 
;
;*************************************************************************************
;*************************************************************************************
;
; Vorbereitungen in jedem Hauptprogramm, das die Bibliotheksroutinen nutzt
;*************************************************************************************
	.equ XTAL = 16000000		; Quarztaktfrequenz in Hertz
	.equ takt = xtal			; for compatibility sake
	.include "m8def.inc"		; Controller Definitionsfile einbinden

.include "i2c_inc_h.asm"		; Header fr die i2c_eeprom Bibliothek
.include "rs232inc_header.asm"	; Header fr die V24-Schnittstelle
.include "arithmetik_header.asm"; Header fr Mathe-Modul

	.equ debuglevel = 0

; Deklarationen
	.equ mega8 	= 1
	.equ CSW 	= SLA_W_24C64	; CHIP-Select fr EEPROM 24C64 auf HW-Adresse 000
	.equ CSR 	= CSW | 0x01	; CSR = CSW mit gesetztem Bit 0
	.equ expandW= SLA_W_PCF8574A; Chipselect fr Portexpander
	.equ expandR= expandW | 0x01; CSR = CSW mit gesetztem Bit 0

	.equ ta = 0x15af
	.equ tar = 0xacc
	.equ testaddr = 0x03A0
; IRQ-Vektoren setzen
;*************************************************************************************
.cseg
.org 0x0000						; RESET-Einsprung und Beginn IRQ-Vektoren
	rjmp start
.org 0x0011						; TWI-IRQ
	rjmp TWI_IRQ	

;*************************************************************************************
.org 0x0013						; Beginn Prgogramm-Speicher
start:
	ldi r16, low(ramend)		; Stackpointer setzen (low)
	out spl, r16
	ldi r16, high(ramend)		; Stackpointer setzen (high)
	out sph, r16

	rcall i2c_init				; I2C-Schnittstelle initialisieren
	ldi r16, i2c_24c64
	sts tw_wait, r16
	rcall serinit				; V24 initialisieren

	ldi zl, low(ss99*2)			; Adresse der Startmeldung via RS232
	ldi zh, high(ss99*2)
	rcall v24_lfcr				; Zeilenvorschub, Wagenrcklauf ausgeben auf RS232		
	rcall v24_flash_string		; Startmeldung via RS232 ausgeben
	rcall v24_lfcr				; Zeilenvorschub, Wagenrcklauf ausgeben auf RS232		

main:	
	sbis ucsra, rxc				; UCSRA = 0x80, wenn ein Zeichen ber RS232 gesendet?
	rjmp main					; wir warten bis ein Byte angekommen ist

JobLoop:
	rcall serin_echo			; Zeichen einlesen und gleich wieder zurcksenden
	cpi r16, 's'				; Zeichen = 's' ?
	breq jl00					; nein, dann die "s"-Sequenz berspringen
	rjmp jl01
jl00:
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, low(ta)
	sts tw_addrl, r16
	ldi r16, high(ta)
	sts tw_addrh, r16
	ldi r16, 0b01110011
	sts tw_data, r16
	rcall i2c_eewrite
	rjmp jl99		
				; 
jl01:
	cpi r16, 'r'				; Zeichen = 'r' ?
	breq jl01a					
	rjmp jl02					; nein, dann die "r"-Sequenz berspringen
jl01a:
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, low(ta)
	sts tw_addrl, r16
	ldi r16, high(ta)
	sts tw_addrh, r16

	rcall i2c_eeread			; Warten auf Byte
	lds r16, tw_data
	rcall v24_number_bin
	rcall v24_lfcr
	rcall v24_lfcr
	rjmp jl99		
	
jl02:
	cpi r16, 'W'				; Zeichen = 'W' ?
	breq jl02a					
	rjmp jl03					; nein, dann die "W"-Sequenz berspringen
jl02a:
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, low(testaddr)
	sts tw_addrl, r16
	ldi r16, high(testaddr)
	sts tw_addrh, r16
	
	ldi r16, 0
	sts tw_data, r16
	.if debuglevel == 1
		rcall v24_lfcr
		rcall v24_number
		ldi r16, ')'
		rcall serout
	.endif
jl02b:
	rcall i2c_eewrite
	.if debuglevel==2
		ldi r16, '$'
		rcall serout
		lds r16, tw_addrh
		rcall v24_number_hex
		lds r16, tw_addrl
		rcall v24_number_hex
		ldi r16, ' '
		rcall serout
	.endif
	rcall i2c_inc_addr
	lds r16, tw_data
	.if debuglevel==2
		ldi r16, '('
		rcall serout
		rcall v24_blank
	.endif
	inc r16
	sts tw_data, r16
	.if debuglevel == 1
		rcall v24_number
		ldi r16, '>'
		rcall serout
	.endif
	lds r16, tw_data
	tst r16
	brne jl02b

	rjmp jl99		
; 
jl03:
	cpi r16, 'R'				; Zeichen = 'R' ?
	breq jl03a					
	rjmp jl04					; nein, dann die "R"-Sequenz berspringen
jl03a:
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, low(testaddr)
	sts tw_addrl, r16
	ldi r16, high(testaddr)
	sts tw_addrh, r16
	ldi r17, 0x00
jl03b:
	rcall i2c_eeread			; Warten auf Byte
	rcall i2c_inc_addr
	lds r16, tw_data
	rcall v24_number
	ldi r16, ' '
	rcall serout
	dec r17
	brne jl03b

	rcall v24_lfcr
	rjmp jl99		

jl04:
	cpi r16, 'L'				; Zeichen = 'L' ?
	breq jl04a					
	rjmp jl05					; nein, dann die "L"-Sequenz berspringen
jl04a:
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, high(last_24c64)
	sts tw_last, r16
	rcall i2c_clear
	rcall v24_done

	rjmp jl99		
; 
jl05:
	cpi r16, 'P'				; Zeichen = 'P' ?
	breq jl05a					
	rjmp jl06					; nein, dann die "P"-Sequenz berspringen
jl05a:
	ldi xl,  low(tw_buffer)
	ldi xh, high(tw_buffer)
	ldi r16, 0
jl05b:
	st x+, r16
	inc r16
	cpi r16, 15
	brne jl05b
	dec r16
	sts tw_buffptr, r16
	ldi r16, csw
	sts tw_hwadr, r16
	ldi r16, low(testaddr+20)
	sts tw_addrl, r16
	ldi r16, high(testaddr+20)
	sts tw_addrh, r16
	rcall i2c_eePwrite
	lds r16, tw_buffptr
	breq jl05c
	rcall v24_done
	rjmp jl99
jl05c:
	rcall v24_error
	rjmp jl99

jl06:
	cpi r16, 'l'				; Zeichen = 'l' ? lschen mit pagewrite
	breq jl06a					
	rjmp jl07					; nein, dann die "l"-Sequenz berspringen
jl06a:
	ldi r16, csw				; Hardwareadresse setzen
	sts tw_hwadr, r16
	ldi r16, 0					; erste Speicherseite 
	sts tw_addrl, r16			; adressieren
	sts tw_addrh, r16
	rcall i2c_erase_all			; 
	rcall v24_done
	rjmp jl99
		


jl07:
	cpi r16, 'a'				; Zeichen = 'a' ? Byte auf Expander
	breq jl07a					
	rjmp jl08					; nein, dann die "a"-Sequenz berspringen
jl07a:
	ldi R16, expandW			; CSW fr Expander laden
	ldi R17, 0xAA				; Bitmuster laden
	rcall i2c_write				; ausgeben
	rjmp jl99

jl08:
	cpi r16, '5'				; Zeichen = '5' ? Byte auf Expander
	breq jl08a					
	rjmp jl09					; nein, dann die "5"-Sequenz berspringen
jl08a:
	ldi R16, expandW			; CSW fr Expander laden
	ldi R17, 0x55				; Bitmuster laden
	rcall i2c_write				; ausgeben
	rjmp jl99

jl09:
	cpi r16, '1'				; Zeichen = '1' ? Byte auf Expander
	breq jl09a					
	rjmp jl10					; nein, dann die "1"-Sequenz berspringen
jl09a:
	ldi R16, expandW			; CSW fr Expander laden
	ldi R17, 0xFF				; Bitmuster laden
	rcall i2c_write				; ausgeben, alle LEDs ein = Bits auf 1
	rjmp jl99

jl10:
	cpi r16, '0'				; Zeichen = '0' ? Byte auf Expander
	breq jl10a					
	rjmp jl11					; nein, dann die "0"-Sequenz berspringen
jl10a:
	ldi R16, expandW			; CSW fr Expander laden
	ldi R17, 0x00				; Bitmuster laden
	rcall i2c_write				; ausgeben, alle LEDs aus = Bits auf 0
	rjmp jl99

jl11:
	cpi r16, 'd'				; Zeichen = 'd' ? Bytes wechselnd auf Expander
	breq jl11a					
	rjmp jl12					; nein, dann die "d"-Sequenz berspringen
jl11a:
	ldi R16, expandW			; CSW fr Expander laden
jl11b:
	ldi R17, 0x55				; Bitmuster laden
	rcall i2c_write				; ausgeben, alle LEDs aus = Bits auf 0
	ldi r17, 0xaa				; Kontramuster laden
	rcall i2c_write				; ausgeben
								; an einem bel. PortPin des PCF8574 (A)
								; kann die Folgefrequenz gemessen werden
								; bei 100 kHz I2C-Frequenz ergibt sich 2,45 kHz
								; (XTAL = 16 MHz)
	rjmp jl11b

jl12:
	cpi r16, 'i'				; Zeichen = 'i' ? Byte von Expander
	breq jl12a					
	rjmp jl13					; nein, dann die "i"-Sequenz berspringen
jl12a:
	ldi R16, expandW			; CSW fr Expander laden
	ldi R17, 0xFF				; Bitmuster fr Eingabe laden
	rcall i2c_write				; ausgeben, alle Bits auf 1
	.if debuglevel == 6
		ldi r16, '#'
		rcall serout
	.endif
	ldi R16, expandR			; CSR fr Expander laden
	rcall i2c_read				; ausgeben, alle LEDs aus = Bits auf 0
	mov r16, r17				; databyte fr Ausgabe auf RS232 vorbereiten
	rcall v24_number_bin		; Binrmuster ausgeben
	rcall v24_lfcr

	rjmp jl99

jl13:
jl99:
	rjmp main
;*************************************************************************************
; Um berschneidung mit den IRQ-Vektoren zu vermeiden, Bibliothek erst hier einbinden
;
.include "i2c_inc.asm"	; Bibliothek einbinden mit enthaltenem IRQ-Handler
; 								;
.include "rs232inc.asm"			; Bibliothek fr die V24-Schnittstelle
;
;*************************************************************************************



ss99:
	.db "I2C-EEPROM-TESTING",0 ; Teststring, nullterminiert
;        012345678901234567





TWI_IRQ:
	reti


.exit

